﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Creator {

  using Common;
  using Configuration;
  using Core;
  using Encodings.IntegerVectorEncoding;
  using Erc;
  using Expressions;
  using Generators.CodeGenerator;

  using HeuristicLab.Data;

  using Persistence.Default.CompositeSerializers.Storable;

  [Item("PointBasedPushProgramCreator", "Creates a Push program whereby points specify the max size but not the min size of the program.")]
  [StorableClass]
  public class PointsBasedPushProgramCreator : UniformRandomIntegerVectorCreator, IPushProgramCreator {
    //private const string ErcOptionsParameterName = "ERC Options";

    public PointsBasedPushProgramCreator(ErcOptions ercOptions) {
      //Parameters.Add(new FixedValueParameter<ErcOptions>(ErcOptionsParameterName, ercOptions));
    }

    [StorableConstructor]
    public PointsBasedPushProgramCreator(bool deserializing) : base(deserializing) { }

    public PointsBasedPushProgramCreator(PointsBasedPushProgramCreator origin, Cloner cloner) : base(origin, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PointsBasedPushProgramCreator(this, cloner);
    }

    public PushProgram CreateProgram(IRandom random, IReadOnlyPushConfiguration config) {
      return LinearCodeGenerator.RandomProgram(LengthParameter.Value.Value, random, config);
    }

    protected override IntegerVector Create(IRandom random, IntValue length, IntMatrix bounds) {
      var x = random.Next(length.Value + 1);
      return base.Create(random, new IntValue(x), bounds);
    }
  }
}
