﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Erc {
  using System.Collections.Generic;
  using System.Linq;

  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Data;
  using HeuristicLab.Parameters;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Erc.Interfaces;

  [StorableClass]
  public class CharErcOptions : ErcOption, IReadOnlyCharErcOptions {
    private const string ConstantsParameterName = "Constants";
    private const string RangeParameterName = "Range";

    public CharErcOptions() : this('a', 'Z') {
    }

    public CharErcOptions(int start, int end, params char[] constants) {
      Parameters.Add(new FixedValueParameter<StringValue>(ConstantsParameterName, new StringValue(new string(constants))));
      Parameters.Add(new FixedValueParameter<IntRange>(RangeParameterName, new IntRange(start, end)));
    }

    [StorableConstructor]
    private CharErcOptions(bool deserializing) : base(deserializing) { }

    public CharErcOptions(CharErcOptions origin, Cloner cloner) : base(origin, cloner) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new CharErcOptions(this, cloner);
    }

    public IValueParameter<StringValue> ConstantsParameter
    {
      get { return (IValueParameter<StringValue>)Parameters[ConstantsParameterName]; }
    }

    public IReadOnlyList<char> Constants
    {
      get { return ConstantsParameter.Value.Value.ToCharArray(); }
      set { ConstantsParameter.Value.Value = new string(value.ToArray()); }
    }

    public IValueParameter<IntRange> RangeParameter
    {
      get { return (IValueParameter<IntRange>)Parameters[RangeParameterName]; }
    }

    public int Start
    {
      get { return RangeParameter.Value.Start; }
      set { RangeParameter.Value.Start = value; }
    }

    public int End
    {
      get { return RangeParameter.Value.End; }
      set { RangeParameter.Value.End = value; }
    }
  }
}
