﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Tells whether the top lit-type item is in the top type vector.
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorContainsExpression<T> : StatelessExpression {
    protected bool Eval(IInternalPushInterpreter interpreter, IPushStack<List<T>> vectorStack, IPushStack<T> literalStack, bool canOverride = false) {
      if (vectorStack.IsEmpty || literalStack.IsEmpty)
        return false;

      var vector = vectorStack.Pop();
      var literal = canOverride ? literalStack.Top : literalStack.Pop();
      var contains = vector.Contains(literal);

      if (canOverride) interpreter.BooleanStack.SetTop(contains);
      else interpreter.BooleanStack.Push(contains);
      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].CONTAINS", StackTypes.Integer | StackTypes.Boolean)]
  public class IntegerVectorContainsExpression : VectorContainsExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.IntegerVectorStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].CONTAINS", StackTypes.Float | StackTypes.Boolean)]
  public class FloatVectorContainsExpression : VectorContainsExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.FloatVectorStack, interpreter.FloatStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].CONTAINS", StackTypes.Boolean)]
  public class BooleanVectorContainsExpression : VectorContainsExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.BooleanVectorStack, interpreter.BooleanStack, true);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].CONTAINS", StackTypes.String | StackTypes.Boolean)]
  public class StringVectorContainsExpression : VectorContainsExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.StringVectorStack, interpreter.StringStack);
    }
  }

}
