﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Pushes a boolean of whether the top vector is empty.
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorEmptyVectorExpression<T> : StatelessExpression {
    protected bool Eval(IInternalPushInterpreter interpreter, IPushStack<List<T>> vectorStack) {
      if (vectorStack.IsEmpty)
        return false;

      var vector = vectorStack.Pop();
      interpreter.BooleanStack.Push(vector.Count == 0);
      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].EMPTYVECTOR", StackTypes.Boolean)]
  public class IntegerVectorEmptyVectorExpression : VectorEmptyVectorExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.IntegerVectorStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].EMPTYVECTOR", StackTypes.Boolean)]
  public class FloatVectorEmptyVectorExpression : VectorEmptyVectorExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.FloatVectorStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].EMPTYVECTOR", StackTypes.Boolean)]
  public class BooleanVectorEmptyVectorExpression : VectorEmptyVectorExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.BooleanVectorStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].EMPTYVECTOR", StackTypes.Boolean)]
  public class StringVectorEmptyVectorExpression : VectorEmptyVectorExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.StringVectorStack);
    }
  }
}
