﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Gets the first item from the type stack
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorFirstExpression<T> : StatelessExpression {
    protected bool Eval(IPushStack<List<T>> vectorStack, IPushStack<T> literalStack) {
      if (vectorStack.IsEmpty || vectorStack.Top.Count == 0) return false;

      var vector = vectorStack.Pop();
      literalStack.Push(vector[0]);
      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].FIRST", StackTypes.Integer)]
  public class IntegerVectorFirstExpression : VectorFirstExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerVectorStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].FIRST", StackTypes.Float)]
  public class FloatVectorFirstExpression : VectorFirstExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatVectorStack, interpreter.FloatStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].FIRST", StackTypes.Boolean)]
  public class BooleanVectorFirstExpression : VectorFirstExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanVectorStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].FIRST", StackTypes.String)]
  public class StringVectorFirstExpression : VectorFirstExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringVectorStack, interpreter.StringStack);
    }
  }
}

