﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using System.Collections.Generic;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Takes the length of the top item on the type stack
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorLengthExpression<T> : StatelessExpression {
    protected bool Eval(IInternalPushInterpreter interpreter, IPushStack<List<T>> vectorStack) {
      if (vectorStack.IsEmpty)
        return false;

      var length = vectorStack.Pop().Count;
      interpreter.IntegerStack.Push(length);
      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].LENGTH", StackTypes.Integer)]
  public class IntegerVectorLengthExpression : VectorLengthExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.IntegerVectorStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].LENGTH", StackTypes.Integer)]
  public class FloatVectorLengthExpression : VectorLengthExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.FloatVectorStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].LENGTH", StackTypes.Integer)]
  public class BooleanVectorLengthExpression : VectorLengthExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.BooleanVectorStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].LENGTH", StackTypes.Integer)]
  public class StringVectorLengthExpression : VectorLengthExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.StringVectorStack);
    }
  }
}
