﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Removes all occurrences of the first lit-type item in the top type vector.
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorRemoveExpression<T> : StatelessExpression {
    protected bool Eval(IPushStack<List<T>> vectorStack, IPushStack<T> literalStack) {
      if (vectorStack.IsEmpty ||
          literalStack.IsEmpty)
        return false;

      var literal = literalStack.Pop();
      vectorStack.Top.RemoveAll(x => x.Equals(literal));
      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].REMOVE", StackTypes.Integer)]
  public class IntegerVectorRemoveExpression : VectorRemoveExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerVectorStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].REMOVE", StackTypes.Float)]
  public class FloatVectorRemoveExpression : VectorRemoveExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatVectorStack, interpreter.FloatStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].REMOVE", StackTypes.Boolean)]
  public class BooleanVectorRemoveExpression : VectorRemoveExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanVectorStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].REMOVE", StackTypes.String)]
  public class StringVectorRemoveExpression : VectorRemoveExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringVectorStack, interpreter.StringStack);
    }
  }
}
