﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Takes the reverse of the top item on the type stack.
  /// </summary>
  /// <param name="vectorStack"></param>
  /// <returns></returns>
  public abstract class VectorReverseExpression<T> : StatelessExpression {
    protected bool Eval(IPushStack<List<T>> vectorStack) {
      if (vectorStack.IsEmpty)
        return false;

      vectorStack.Top.Reverse();
      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].REVERSE")]
  public class IntegerVectorReverseExpression : VectorReverseExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerVectorStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].REVERSE")]
  public class FloatVectorReverseExpression : VectorReverseExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatVectorStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].REVERSE")]
  public class BooleanVectorReverseExpression : VectorReverseExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanVectorStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].REVERSE")]
  public class StringVectorReverseExpression : VectorReverseExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringVectorStack);
    }
  }
}
